<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 01/08/2016
 * Time: 14:57
 */

namespace Service;


use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\Log\Exception;

class Classification
{
    /** @var Repositories\Classification */
    protected $classificationRepository;

    /**
     * @var EntityManager
     */
    private $entityManager;
    /**
     * @var Utilities
     */
    private $environment;
    /**
     * @var \Factory\Classification
     */
    private $classificationFactory;
    /**
     * @var ClassificationProjectType
     */
    private $classificationProjectTypeService;
    /**
     * @var ClassificationTaskStatus
     */
    private $classificationTaskStatusService;

    public function __construct(EntityManager $entityManager, \Util\Utilities $environment,
                                \Factory\Classification $classificationFactory, ClassificationProjectType $classificationProjectTypeService,
                                ClassificationTaskStatus $classificationTaskStatusService)
    {

        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->classificationFactory = $classificationFactory;
        $this->classificationRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Classification');
        $this->classificationProjectTypeService = $classificationProjectTypeService;
        $this->classificationTaskStatusService = $classificationTaskStatusService;
    }

    public static function getArrayControlForHomologation()
    {
        return array(
            Entities\Classification::CONTROL_TEST,
            Entities\Classification::CONTROL_ERROR_TEST,
            Entities\Classification::CONTROL_CONFLICT,
            Entities\Classification::CONTROL_CONFLICT_DEVOPS,
            Entities\Classification::CONTROL_MERGE_REQUEST,
            Entities\Classification::CONTROL_ERROR_MERGE_REQUEST,
            Entities\Classification::CONTROL_REPLICATION,
        );
    }
    
    public static function getArrayAssociativeByControl()
    {
        return array(
                "C"=> "Conflito",
                "D"=> "Conflito (DevOps)",
                "E"=> "Erro de teste",
                "G"=> "Erro (Merge Request)",
                "M"=> "Merge Request",
                "R"=> "Replicação de código",
                "T"=> "Teste",
        );
    }

    public function saveClassification($id, $label, $isProject, $isTask, $control,$arrayAllowedProjectTypeId = array(),
                                       $arrayAllowedProjectStatusId = array(), $arrayNotAllowedProjectTypeId = array(),
                                       $arrayNotAllowedProjectStatusId = array(), $estimatedTimeProportion = null,
                                       $divideDuration = 'N', $usedInTimeControl = 'Y', $accessProfile = array(), $usedInNews = 'N')
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $classification = null;
            if (!empty($id)) {
                $classification = $this->classificationRepository->find($id);
            }
            $classification = $this->classificationFactory->saveClassification($classification, $isProject, $label, $isTask,$control, $estimatedTimeProportion, $usedInTimeControl, $usedInNews);
            if($divideDuration){
                $classification->setDivideDuration($divideDuration);
            }
            $this->entityManager->persist($classification);
            $this->entityManager->flush();

            $this->classificationTaskStatusService->saveArrayAllowedTaskStatusId($classification, $arrayAllowedProjectStatusId);
            $this->classificationTaskStatusService->saveArrayNotAllowedTaskStatusId($classification, $arrayNotAllowedProjectStatusId);

            $this->classificationProjectTypeService->saveArrayAllowedProjectTypeId($classification, $arrayAllowedProjectTypeId);
            $this->classificationProjectTypeService->saveArrayNotAllowedProjectTypeId($classification, $arrayNotAllowedProjectTypeId);

            $this->saveClassificationAccessProfile($classification, $accessProfile);

            $connection->commit();
            
            return array("ID" => $classification->getId());
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param \Zeedhi\DevOps\Model\Projectmanagement\Entities\Classification $classification
     * @param $accessProfile
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function saveClassificationAccessProfile($classification, $accessProfile)
    {
        $classificationPerfilacessoRepository = InstanceProvider::getRepository()->getClassificationPerfilacesso();
        $classificationPerfilaceesos = $classificationPerfilacessoRepository->findBy(array('classification' => $classification->getId()));
            foreach ($classificationPerfilaceesos as $c){
                $this->entityManager->remove($c);
            }

            foreach ($accessProfile as $profile){
                $classificationPerfilaceeso = InstanceProvider::getFactory()->getClassificationPerfilacesso()->createClassificationPerfilacesso($classification, $profile);
                $this->entityManager->persist($classificationPerfilaceeso);
            }
            $this->entityManager->flush();
    }

    public function findClassifications($conditions)
    {
        $organizationId = $this->environment->getCurrentOrganizationId();
        $classifications = $this->classificationRepository->findClassifications($conditions,$organizationId);
        foreach ($classifications as &$classification) {
            $classification['ALLOWED_PROJECT_STATUS'] = Utilities::setSelectMultiInQuery($classification['ALLOWED_PROJECT_STATUS']);
            $classification['ALLOWED_PROJECT_TYPE'] = Utilities::setSelectMultiInQuery($classification['ALLOWED_PROJECT_TYPE']);
            $classification['NOT_ALLOWED_PROJECT_STATUS'] = Utilities::setSelectMultiInQuery($classification['NOT_ALLOWED_PROJECT_STATUS']);
            $classification['NOT_ALLOWED_PROJECT_TYPE'] = Utilities::setSelectMultiInQuery($classification['NOT_ALLOWED_PROJECT_TYPE']);
            $classification['NRPERFILACESSO'] = Utilities::setSelectMultiInQuery($classification['NRPERFILACESSO']);
        }
        return $classifications;
    }

    public function findClassificationForTask($projectStatus,$projectType, $filterString = null)
    {
        $filterString = Utilities::createQueryFiltered($filterString);
        $organizationId = $this->environment->getCurrentOrganizationId();
        $classifications = $this->classificationRepository->findClassificationForTask($projectStatus,$projectType,$organizationId, $filterString);
        return $classifications;
    }
    public function findClassificationForMeeting($projectId, $filterString = null)
    {
        $classifications = array();
        if($projectId){
            $filterString = Utilities::createQueryFiltered($filterString);
            $organizationId = $this->environment->getCurrentOrganizationId();
            list($projectStatus,$projectType) = InstanceProvider::getRepository()->getProject()->getProjectStatusAndType($projectId);
            $classifications = $this->classificationRepository->findClassificationForTask($projectStatus,$projectType,$organizationId, $filterString);
        }
        return $classifications;
    }

    public function deleteClassification($classificationId)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            /** @var Entities\Classification $classification */
            $classification = $this->classificationRepository->find($classificationId);
            $this->classificationProjectTypeService->deleteClassificationProjectType($classification);
            $this->classificationTaskStatusService->deleteClassificationTaskStatus($classification);
            $this->entityManager->remove($classification);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Doctrine\DBAL\DBALException $e) {
            $connection->rollBack();
            throw new \Exception($e->getMessage(), 1);
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }

    }

    /**
     * @param $taskClassificationControl
     * @return string
     */
    public function getAllowedAccessProfileFilter($taskClassificationControl)
    {

        $classificationIds = array();
        $classificationPerfilacessoIds = array();
        $classifications = InstanceProvider::getRepository()->getClassification()->findBy(array('control' => $taskClassificationControl));
        foreach ($classifications as $c) {
            array_push($classificationIds, $c->getId());
        }
        $classificationPerfilacesso = InstanceProvider::getRepository()->getClassificationPerfilacesso()->findBy(array('classification' => $classificationIds));
        foreach ($classificationPerfilacesso as $cp){
           array_push($classificationPerfilacessoIds, $cp->getNrperfilacesso());
        }
        $classificationPerfilacessoIds = implode(',', $classificationPerfilacessoIds);
        if($classificationPerfilacessoIds){
            return 'AND PER.NRPERFILACESSO IN (' . $classificationPerfilacessoIds . ')';
        }
        return '';
    }
} 