<?php
namespace Zeedhi\DevOps\Model\Servermanager\Repositories;


use Doctrine\DBAL\Platforms\SQLAnywhere11Platform;
use Zeedhi\DevOps\Model\Configurationmanager\Entities;

class DeployServer extends \Doctrine\ORM\EntityRepository {
    /**
     * @param $deployId
     * @param $prodServerIds
     * @param $filterDeployServerStatus
     * @param $filterProductServer
     * @param $organizationId
     * @return array
     */

    public function verifyActiveServers($filters){
        $query = <<<SQL
          SELECT SR.NAME
          FROM PRODUCT_SERVER PS
          JOIN SERVER SR ON PS.SERVER_ID = SR.ID
          WHERE PS.IS_ACTIVE = 'N'
          $filters
          GROUP BY SR.NAME
SQL;

        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }

    public function findServerInformationDeploy($deployId, $prodServerIds, $filterDeployServerStatus,
                                                $filterProductServer,$organizationId) {
        $query = <<<SQL
			SELECT PS.ID AS PRODUCT_SERVER_ID, PS.PATH AS PROD_BASE_PATH,
			       DS.DEPLOYMENT_ORDER, SR.ID, SR.NAME, SR.WSEMAIL,
			       NVL(PS.BACKUP_LIMIT, SR.BACKUP_LIMIT) AS BACKUP_LIMIT,
			       DS.ID AS DEPLOY_SERVER_ID, PS.IS_ACTIVE
              FROM PRODUCT_SERVER PS
              LEFT JOIN DEPLOY_SERVER DS ON DS.PRODUCT_SERVER_ID = PS.ID AND DS.DEPLOY_ID = :DEPLOY_ID
              LEFT JOIN SERVER SR ON SR.ID = PS.SERVER_ID
             WHERE PS.ORGANIZATION_ID = :ORGANIZATION
               AND PS.ID IN ($prodServerIds)
               $filterDeployServerStatus
               $filterProductServer
             ORDER BY DS.DEPLOYMENT_ORDER
SQL;
        $params = array(
            'DEPLOY_ID' => $deployId,
            'ORGANIZATION' => $organizationId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findOldDeployServerToInactive($productId, $productServerId, $deployId) {
        $query = <<<SQL
			SELECT DE.ID AS DEPLOY_ID, DS.ID AS DEPLOY_SERVER_ID,
                   TO_CHAR(DE.DEPLOY_DATE, 'DD/MM/YYYY HH24:MI:SS') AS DEPLOY_DATE
              FROM DEPLOY DE
             INNER JOIN DEPLOY_SERVER DS ON DS.DEPLOY_ID = DE.ID
             WHERE DE.PRODUCT_ID = :PRODUCT_ID
               AND DE.IS_ACTIVE = :IS_ACTIVE_YES
               AND DS.IS_ACTIVE = :IS_ACTIVE_YES
               AND DS.PRODUCT_SERVER_ID = :PRODUCT_SERVER_ID
               AND DS.STATUS = :DEPLOYED_STATUS
               AND DE.ID <> :DEPLOY_ID
             ORDER BY DE.DEPLOY_DATE DESC
SQL;
        $params = array(
            'PRODUCT_ID' => $productId,
            'IS_ACTIVE_YES' => Entities\Deploy::IS_ACTIVE_YES,
            'PRODUCT_SERVER_ID' => $productServerId,
            'DEPLOYED_STATUS' => Entities\Deploy::DEPLOYED_DEPLOY_STATUS,
            'DEPLOY_ID' => $deployId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
}